@@ -33,14 +33,15 @@ class SearchClient(NamespacedClient):
         return DataIndex(index, self)
 
 
-class Response:
+class BaseResponse:
     def __init__(self, data: dict):
         self.response = data
 
     def __getitem__(self, key):
         return self.response.get(key)
 
-    @property
+
+class SearchResponse(BaseResponse):
     def hits_total(self) -> Optional[int]:
         total = self['hits'].get('total')
         if total:
@@ -50,7 +51,7 @@ class Response:
         return [h['_source'] for h in self['hits']['hits']]
 
     def raw_hits(self) -> List[dict]:
-        return self['hits']
+        return self['hits']['hits']
 
     def count(self) -> int:
         return self['count']
@@ -59,6 +60,11 @@ class Response:
         return self['aggregations'][name]['buckets']
 
 
+class BulkResponse(BaseResponse):
+    def errors(self) -> bool:
+        return self['errors']
+
+
 class DataIndex:
     """
     索引
@@ -76,7 +82,7 @@ class DataIndex:
         """
 
         resp = self._client.session.head(
-            f'{self._client.api_prefix}/{self._index}/{id}',
+            f'{self._client.api_prefix}/indices/{self._index}/data/{id}',
         )
         if resp.status_code == 404:
             return False
@@ -92,70 +98,86 @@ class DataIndex:
 
         data = read_json_data(
             self._client.session.get(
-                f'{self._client.api_prefix}/{self._index}/{id}',
+                f'{self._client.api_prefix}/indices/{self._index}/data/{id}',
             )
         )
         return data['_source']
 
-    def put(self, id: str, data: dict):
+    def put(self, id: str, data: dict, refresh: bool = None):
         """
         更新指定数据，指定ID不存在时自动创建数据
 
         :param id: 数据ID
         :param data: 数据内容
+        :param refresh: 强制刷新索引
         """
 
+        args = {
+            'body': data,
+        }
+        if refresh is not None:
+            args['refresh'] = refresh
         read_json_data(
             self._client.session.put(
-                f'{self._client.api_prefix}/{self._index}/{id}',
-                data=_json.dumps({
-                    'data': data,
-                }).encode(),
+                f'{self._client.api_prefix}/indices/{self._index}/data/{id}',
+                data=_json.dumps(args).encode(),
                 headers={
                     'content-type': _json.mimetype,
                 },
             )
         )
 
-    def update(self, id: str, data: str = None):
+    def update(self, id: str, data: str = None, refresh: bool = None):
         """
         更新数据的指定字段
 
         :param id: 数据ID
         :param data: 更新的数据内容
+        :param refresh: 强制刷新索引
         """
 
+        args = {
+            'body': {
+                'doc': data,
+            },
+        }
+        if refresh is not None:
+            args['refresh'] = refresh
         read_json_data(
             self._client.session.put(
-                f'{self._client.api_prefix}/{self._index}/update/{id}',
-                data=_json.dumps({
-                    'body': {
-                        'doc': data,
-                    },
-                }).encode(),
+                f'{self._client.api_prefix}/indices/{self._index}/update/{id}',
+                data=_json.dumps(args).encode(),
                 headers={
                     'content-type': _json.mimetype,
                 },
             )
         )
 
-    def delete(self, id: str):
+    def delete(self, id: str, refresh: bool = None):
         """
         删除指定数据
 
         :param id: 数据ID
+        :param refresh: 强制刷新索引
         """
 
+        args = {}
+        if refresh is not None:
+            args['refresh'] = refresh
         read_json_data(
             self._client.session.delete(
-                f'{self._client.api_prefix}/{self._index}/{id}',
+                f'{self._client.api_prefix}/indices/{self._index}/data/{id}',
+                data=_json.dumps(args).encode(),
+                headers={
+                    'content-type': _json.mimetype,
+                },
             )
         )
 
-    def search_by_dsl(self, query_body: dict) -> 'Response':
+    def search_by_dsl(self, query_body: dict) -> 'SearchResponse':
         data = read_json_data(
             self._client.session.post(
-                f'{self._client.api_prefix}/{self._index}/search-by-dsl',
+                f'{self._client.api_prefix}/indices/{self._index}/search-by-dsl',
                 data=_json.dumps({
                     'body': query_body,
                 }).encode(),
@@ -164,12 +186,12 @@ class DataIndex:
                 },
             )
         )
-        return Response(data)
+        return SearchResponse(data)
 
-    def count_by_dsl(self, query_body: dict) -> 'Response':
+    def count_by_dsl(self, query_body: dict) -> 'SearchResponse':
         data = read_json_data(
             self._client.session.post(
-                f'{self._client.api_prefix}/{self._index}/count-by-dsl',
+                f'{self._client.api_prefix}/indices/{self._index}/count-by-dsl',
                 data=_json.dumps({
                     'body': query_body,
                 }).encode(),
@@ -178,7 +200,38 @@ class DataIndex:
                 },
             )
         )
-        return Response(data)
+        return SearchResponse(data)
+
+    def update_by_dsl(self, query_body: dict) -> 'BaseResponse':
+        data = read_json_data(
+            self._client.session.post(
+                f'{self._client.api_prefix}/indices/{self._index}/update-by-dsl',
+                data=_json.dumps({
+                    'body': query_body,
+                }).encode(),
+                headers={
+                    'content-type': _json.mimetype,
+                },
+            )
+        )
+        return BaseResponse(data)
+
+    def bulk(self, actions: List[dict], refresh: bool = None):
+        data = {
+            'actions': actions,
+        }
+        if refresh is not None:
+            data['refresh'] = refresh
+        data = read_json_data(
+            self._client.session.post(
+                f'{self._client.api_prefix}/indices/{self._index}/bulk',
+                data=_json.dumps(data).encode(),
+                headers={
+                    'content-type': _json.mimetype,
+                },
+            )
+        )
+        return BulkResponse(data)
 
     def query(self, name: str = None) -> 'QueryBuilder':
         """
